﻿using System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using System.Threading.Tasks;
using System.Threading.Tasks.Dataflow;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace VSMDataFlowPart2
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        BroadcastBlock<int> _broadcaster;

        public MainPage()
        {
            this.InitializeComponent();
        }

        private ActionBlock<T> CreateUiUpdateActionBlock<T>(TextBlock element)
        {
            return new ActionBlock<T>(x =>
                element.Text = x.ToString(),
                  new ExecutionDataflowBlockOptions() { TaskScheduler = TaskScheduler.FromCurrentSynchronizationContext() });
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            _broadcaster = new BroadcastBlock<int>(x => x);
        }

        private async void BroadCast_Click(object sender, RoutedEventArgs e)
        {
            ActionBlock<int> simpleDisplayer = CreateUiUpdateActionBlock<int>(Message);
            _broadcaster.LinkTo(simpleDisplayer);
            await BroadCastData();
        }

        private async Task BroadCastData()
        {
            Random r = new Random((int)System.DateTime.Now.Ticks);
            int temp = 0;

            for (int i = 0; i < 1000; i++)
            {
                await Task.Delay(125);
                temp = r.Next(60, 80);
                await _broadcaster.SendAsync(temp);
            }
        }

        private void Transform_Click(object sender, RoutedEventArgs e)
        {
            TransformBlock<int, string> formatter = new TransformBlock<int, string>(x =>
            String.Format("{0:G2}°F", x));
            ActionBlock<string> transformedDisplayed = CreateUiUpdateActionBlock<string>(TransformedMessage);
            _broadcaster.LinkTo(formatter);
            formatter.LinkTo(transformedDisplayed);
        }

        private void PostProcess_Click(object sender, RoutedEventArgs e)
        {
            const int average = 75;
            TransformBlock<int, int> averageDelta = new TransformBlock<int, int>(x => x - average);
            TransformBlock<int, string> formatPost = new TransformBlock<int, string>(x =>
            {
                string preFix = string.Empty;
                if (x > 0)
                    preFix = "+";

                return String.Format("{0}{1:G2}°F", preFix, x);
            });

            ActionBlock<string> postProcessDisplayer = CreateUiUpdateActionBlock<string>(PostProcessMessage);

            _broadcaster.LinkTo(averageDelta);
            averageDelta.LinkTo(formatPost);
            formatPost.LinkTo(postProcessDisplayer);
        }
    }
}
